library static_sharedpreferences;

import 'package:shared_preferences/shared_preferences.dart';

///For static use of the SharedPreferences Instance
class SP {
  static SharedPreferences _sp;

  /// ## Description
  ///
  /// * You have to put this method in your main(), like in the example down below.
  ///
  ///
  /// * Loads and parses the [SharedPreferences] for this app from disk.
  ///   Because this is reading from disk, it shouldn't be awaited in
  ///   performance-sensitive blocks.
  ///
  /// ## Example
  /// {@tool sample}
  /// ```dart
  /// void main() {
  ///   WidgetsFlutterBinding.ensureInitialized();
  ///
  ///   SP.initialize().whenComplete((){
  ///     print('Done with initialization!');
  ///   });
  /// }
  ///
  /// ```
  /// {@end-tool}
  static Future initialize() async =>
      _sp = await SharedPreferences.getInstance();

  static SharedPreferences get sharedPrefs =>
      (_sp != null) ? _sp : onSharedPreferencesIsNull();

  /// Returns all keys in the persistent storage.
  static Set<String> getKeys() =>
      (_sp != null) ? _sp.getKeys() : onSharedPreferencesIsNull();

  /// Reads a value from persistent storage, throwing an exception if it's not an int.
  static int getInt(String key) =>
      (_sp != null) ? _sp.getInt(key) : onSharedPreferencesIsNull();

  /// Saves an integer [value] to persistent storage in the background.
  /// If [value] is null, this is equivalent to calling [remove()] on the [key].
  static Future<bool> setInt(String key, int value) =>
      (_sp != null) ? _sp.setInt(key, value) : onSharedPreferencesIsNull();

  /// Reads a value from persistent storage, throwing an exception if it's not a double.
  static double getDouble(String key) =>
      (_sp != null) ? _sp.getDouble(key) : onSharedPreferencesIsNull();

  /// Saves a double [value] to persistent storage in the background.
  ///
  /// Android doesn't support storing doubles, so it will be stored as a float.
  ///
  /// If [value] is null, this is equivalent to calling [remove()] on the [key].
  static Future<bool> setDouble(String key, double value) =>
      (_sp != null) ? _sp.setDouble(key, value) : onSharedPreferencesIsNull();

  /// Reads a value from persistent storage, throwing an exception if it's not a String.
  static String getString(String key) =>
      (_sp != null) ? _sp.getString(key) : onSharedPreferencesIsNull();

  /// Saves a string [value] to persistent storage in the background.
  ///
  /// If [value] is null, this is equivalent to calling [remove()] on the [key].
  static Future<bool> setString(String key, String value) =>
      (_sp != null) ? _sp.setString(key, value) : onSharedPreferencesIsNull();

  /// Reads a set of string values from persistent storage, throwing an exception if it's not a string set.
  static List<String> getStringList(String key) =>
      (_sp != null) ? _sp.getStringList(key) : onSharedPreferencesIsNull();

  /// Saves a list of strings [value] to persistent storage in the background.
  ///
  /// If [value] is null, this is equivalent to calling [remove()] on the [key].
  static Future<bool> setStringList(String key, List<String> value) =>
      (_sp != null)
          ? _sp.setStringList(key, value)
          : onSharedPreferencesIsNull();

  /// Reads a value from persistent storage, throwing an exception if it's not a bool.
  static bool getBool(String key) =>
      (_sp != null) ? _sp.getBool(key) : onSharedPreferencesIsNull();

  /// Saves a boolean [value] to persistent storage in the background.
  ///
  /// If [value] is null, this is equivalent to calling [remove()] on the [key].
  static Future<bool> setBool(String key, bool value) =>
      (_sp != null) ? _sp.setBool(key, value) : onSharedPreferencesIsNull();

  /// Reads a value of any type from persistent storage.
  static dynamic getDynamic(String key) =>
      (_sp != null) ? _sp.get(key) : onSharedPreferencesIsNull();

  /// ## Description
  ///
  /// * This method gets triggered if you try to use a method,
  ///   which required the SharedPreferences instance, but
  ///   the instance == null
  ///
  ///
  /// * If not changed by the user, then this will just use
  ///   the print() function to notify the developer
  ///   of the state of the SharedPreferences instance.
  ///
  /// ## Example
  ///
  /// {@tool sample}
  /// ```dart
  /// //some code...
  ///
  /// int value = SP.getInt(myKey);
  /// //You try to restore an integer,
  /// //but in case SP.sharedPrefs == null
  /// //then onSharedPreferencesIsNull() gets executed
  ///
  /// //some code...
  /// ```
  /// {@end-tool}
  static Function onSharedPreferencesIsNull = () {
    print("\n\n\n" + "SharedPreferences is null $_sp" + "\n\n\n");
  };
}

/// ## Description
/// 
/// * Can be ideally used for switch- or if-statements
///
/// ## Example
/// 
/// {@tool sample}
/// ```dart
/// switch (mySpType) {
///   case SP_Type.BOOL:
///     SP.setBool(myKey, true);
///     break;
///   case SP_Type.INT:
///     int cache = SP.getInt(myKey) ?? 0;
///     SP.setInt(myKey, cache + 1);
///     break;
///   default:
/// }
///
/// ```
///
/// {@end-tool}
enum SP_Type {
  INT,
  DOUBLE,
  STRING,
  BOOL,
  DYNAMIC,
  LIST_STRING,
}
